import { ApiHeaders, RequestBody, ResponseBody } from '../api/typings';
export declare type ApiQueryParams = Record<string, string | number | boolean>;
export interface EndpointCreateRequestInit {
    query?: ApiQueryParams;
    headers?: ApiHeaders;
    body?: RequestBody;
    extraKey?: string;
}
export declare type ApiParam = string | number;
/**
 * `HttpEndpoints` provides static utilities for implementing a set of
 * API endpoints which returns requests that can be passed to `api.request`.
 */
export declare class HttpEndpoints {
    /**
     * The base path to prefix all underlying requests.
     */
    protected static basePath: string;
    /**
     * Some REST APIs require that requests end with a trailing slash. (ie. `/users/` instead of `/users`)
     * If `true`, will ensure that a trailing `/` is added to each request.
     */
    protected static trailingSlash: boolean;
    /**
     * Returns a new path by appending a given `relativePath` to `this.basePath`.
     */
    protected static buildPath(relativePath: string): string;
    /**
     * Creates a `GET` request.
     *
     * @param path A path relative to the base path.
     * @param requestInit Additional request parameters.
     */
    protected static _get<TResponseBody extends ResponseBody = ResponseBody>(path: string, requestInit?: Omit<EndpointCreateRequestInit, 'body'>): import("../api/typings").ApiGetDeleteRequestParams<"GET", TResponseBody>;
    /**
     * Creates a `POST` request.
     *
     * @param path A path relative to the base path.
     * @param requestInit Additional request parameters.
     */
    protected static _post<TResponseBody extends ResponseBody = ResponseBody>(path: string, requestInit?: EndpointCreateRequestInit): import("../api/typings").ApiMutationRequestParams<"POST", TResponseBody>;
    /**
     * Creates a `PUT` request.
     *
     * @param path A path relative to the base path.
     * @param requestInit Additional request parameters.
     */
    protected static _put<TResponseBody extends ResponseBody = ResponseBody>(path: string, requestInit?: EndpointCreateRequestInit): import("../api/typings").ApiMutationRequestParams<"PUT", TResponseBody>;
    /**
     * Creates a `PATCH` request.
     *
     * @param path A path relative to the base path.
     * @param requestInit Additional request parameters.
     */
    protected static _patch<TResponseBody extends ResponseBody = ResponseBody>(path: string, requestInit?: EndpointCreateRequestInit): import("../api/typings").ApiMutationRequestParams<"PATCH", TResponseBody>;
    /**
     * Creates a `DELETE` request.
     *
     * @param path A path relative to the base path.
     * @param requestInit Additional request parameters.
     */
    protected static _delete<TResponseBody extends ResponseBody = ResponseBody>(path: string, requestInit?: EndpointCreateRequestInit): import("../api/typings").ApiGetDeleteRequestParams<"DELETE", TResponseBody>;
    /**
     * Given an object of key-value pairs, will serialize the query parameters into a search string.
     * This is called internally when serializing the query parameters into the request `url`.
     *
     * To customize query parameter serialization, override this in your base class.
     */
    protected static _serializeQuery(query: ApiQueryParams): string;
    private static _createRequest;
}
export declare class RestEndpoints extends HttpEndpoints {
    protected static _collectionPath: string;
    protected static _singlePath(id: ApiParam): string;
    /**
     * Returns the path for a given resource id, as a convenience.
     */
    static pathToResource(id: ApiParam): string;
    /**
     * Performs a `GET` at `/`, appending an optional `query`
     * @param query Query parameters
     */
    protected static _list<TResponseBody extends ResponseBody = ResponseBody>(query?: ApiQueryParams): import("../api/typings").ApiGetDeleteRequestParams<"GET", TResponseBody>;
    /**
     * Performs a `POST` at `/`, using an optional request `body`
     */
    protected static _create<TResponseBody extends ResponseBody = ResponseBody>(body?: RequestBody): import("../api/typings").ApiMutationRequestParams<"POST", TResponseBody>;
    /**
     * Performs a `GET` at `/:id`
     */
    protected static _findById<TResponseBody extends ResponseBody = ResponseBody>(id: ApiParam): import("../api/typings").ApiGetDeleteRequestParams<"GET", TResponseBody>;
    /**
     * Performs a `PUT` at `/:id`, using an optional request `body`
     */
    protected static _update<TResponseBody extends ResponseBody = ResponseBody>(id: ApiParam, body?: RequestBody): import("../api/typings").ApiMutationRequestParams<"PUT", TResponseBody>;
    /**
     * Performs a `PATCH` at `/:id`, using an optional request `body`
     */
    protected static _partialUpdate<TResponseBody extends ResponseBody = ResponseBody>(id: ApiParam, body?: RequestBody): import("../api/typings").ApiMutationRequestParams<"PATCH", TResponseBody>;
    /**
     * Performs a `DELETE` at `/:id`, using an optional request `body`
     */
    protected static _destroy(id: ApiParam, body?: RequestBody): import("../api/typings").ApiGetDeleteRequestParams<"DELETE", ResponseBody>;
}
/**
 * Applies colon-delimited path params to a given path pattern
 *
 * @param pattern Path pattern
 * @param params Path params
 *
 * @example `buildPath('/users/:id', { id: 4 })` => `/users/4`
 */
export declare function dynamicPath<TParamKey extends string>(pattern: string, params: Record<TParamKey, ApiParam>): string;
