import { ApiRequestFetchPolicy, ApiRequestMethod, ApiRequestParams, ResponseBody } from '../../api/typings';
interface UseApiQueryData<TResponseBody extends ResponseBody> {
    data: TResponseBody | undefined | null;
    loading: boolean;
    error: Error | null;
}
export interface UseApiQueryActions<TResponseBody extends ResponseBody> {
    /**
     * Sets the response data of the Api request manually.
     *
     * If this request is already reading or writing to the cache,
     * it will update the underlying cache value.
     *
     * This is useful if you mutate the data via a separate API call, and then and want to manually
     * update the data in this call.
     */
    setData: (data: TResponseBody) => void;
    /**
     * Performs a manual API fetch (over the network).
     *
     * Note: If fetch-policy is set to `no-cache`, it will not
     * persist to the cache.
     */
    refetch: (opts?: {
        /**
         * Sets `data` to `null` when the request kicks off.
         */
        reinitialize?: boolean;
        /**
         * Passed as an option to `api.request` and guarantees that a
         * new request will be made.
         *
         * By default, it will use `deduplicate` from the hook's options
         * (`true` for `GET` requests, `false` for non-`GET` requests).
         */
        deduplicate?: boolean;
    }) => void;
}
/**
 * API hook to run an api `GET` request, returning a `Loadable`
 * instance of the response data
 */
export declare function useApiQuery<TResponseBody extends ResponseBody>(params: ApiRequestParams<ApiRequestMethod, TResponseBody> | null, opts?: {
    fetchPolicy?: ApiRequestFetchPolicy;
    deduplicate?: boolean;
    /**
     * If true, will keep `data` from previous requests
     * until new data is received.
     */
    dontReinitialize?: boolean;
}): [UseApiQueryData<TResponseBody>, UseApiQueryActions<TResponseBody>];
export {};
